package org.jboss.soa.qa.drools.cep

import javax.security.auth.Subject
import javax.security.auth.callback.Callback
import javax.security.auth.callback.CallbackHandler
import javax.security.auth.callback.NameCallback
import javax.security.auth.callback.PasswordCallback
import javax.security.auth.callback.UnsupportedCallbackException
import javax.security.auth.login.LoginContext

import org.jboss.security.SecurityContextAssociation
import org.jboss.security.SecurityContextFactory
import org.jboss.security.SimplePrincipal
import org.jboss.security.identity.plugins.SimpleRoleGroup
import org.jboss.shrinkwrap.api.spec.JavaArchive
import org.jboss.soa.qa.shared.data.OrderAck
import org.jboss.soa.qa.shared.data.SimpleOrder

import switchyard.qa.audit.ProviderAuditor.Property
import switchyard.qa.audit.common.AuditChecker

import common.qa.packaging.PrepareArchive

public class TestCommons {

	def static JavaArchive createDeployment() {
		return PrepareArchive.fromProjectNameWithClasses(
		'drools-cep-test',
		TestCommons.class,
		org.jboss.soa.qa.shared.service.LoggingService.class,
		org.jboss.soa.qa.shared.service.LoggingServiceBean.class,
		org.jboss.soa.qa.shared.service.OrderService.class,
		org.jboss.soa.qa.shared.service.OrderServiceBean.class,
		org.jboss.soa.qa.shared.service.InventoryService.class,
		org.jboss.soa.qa.shared.service.InventoryServiceBean.class,
		org.jboss.soa.qa.shared.service.OrderService.class,
		org.jboss.soa.qa.shared.data.SimpleOrder.class,
		org.jboss.soa.qa.shared.data.OrderItem.class,
		org.jboss.soa.qa.shared.data.OrderAck.class,
		org.jboss.soa.qa.shared.data.OrderException.class
		)
	}

	def static login() {
		def lc = new LoginContext('other', new CallbackHandler() {
					public void handle(Callback[] cb) throws IOException, UnsupportedCallbackException {
						cb.each {
							switch (it.class) {
								case NameCallback:
									it.name = 'userfriend'
									break
								case PasswordCallback:
									it.password = 'USERFRIEND.1'.chars
									break
								default:
									throw new UnsupportedCallbackException(it)
							}
						}
					};
				})
		lc.login()

		def username = new SimplePrincipal('userfriend')
		def subject = new Subject()
		subject.principals.add(username)
		def sc = SecurityContextFactory.createSecurityContext(username, 'USERFRIEND.1', subject, 'other')
		def group = new SimpleRoleGroup('friend')
		sc.getUtil().setRoles(group)
		SecurityContextAssociation.setSecurityContext(sc)
		SecurityContextAssociation.securityContext = SecurityContextFactory.createSecurityContext(lc.subject.principals.first(), 'USERFRIEND.1', lc.subject, 'other')
	}
	
	def static logout() {
		SecurityContextAssociation.clearSecurityContext()
	}
	
	def static getRegistry(beanService) {
		def methodHandlerField = beanService.class.getDeclaredField("methodHandler")
		methodHandlerField.accessible = true
		def methodHandler = methodHandlerField.get(beanService)

		def beanField = methodHandler.class.getDeclaredField("bean")
		beanField.accessible = true
		def bean = beanField.get(methodHandler)

		def serviceField = bean.class.getDeclaredField("_service")
		serviceField.accessible = true
		def service = serviceField.get(bean)
		
		service.domain.serviceRegistry
	}
	
	def static cleanRegistry(registry) {
		println "Cleaning from $registry"
		registry.serviceReferences.each {
			println "Removing reference $it"
			registry.unregisterServiceReference(it)
		}
	}
}

